#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

typedef struct _tree_node{
	int value;
	int count;
	struct _tree_node *left;
	struct _tree_node *right;
} treeNode;

typedef struct{
	void *mem;
	int stacksize;
	int stackpos;
} stack;

treeNode *new(int val){
	treeNode *p = malloc(sizeof(treeNode));
	assert(p != NULL);
	p->value = val;
	p->count = 1;
	p->left = NULL;
	p->right = NULL;
	return p;
}

treeNode *add(int val, treeNode *root){
	if(root == NULL) 
		return new(val);
	treeNode *p = root;
	while(1){
		if(val == p->value){
			p->count++; //nothing is added
			break;
		}else if(val < p->value){
			if(p->left == NULL){
				p->left = new(val);
				break;
			} else
			p = p->left;	
		}else if(val > p->value){
			if(p->right == NULL){
				p->right = new(val);
				break;
			} else
			p = p->right;	
		}
	}
	return root;
}

void stackinit(stack *st){
	st->stacksize = 20;
	st->stackpos = 0;
	st->mem=malloc(st->stacksize);
	assert(st->mem != NULL);
}

void push(stack *st, void *data, int size){
	if(st->stackpos + size > st->stacksize){
		st->stacksize = 2 * st->stacksize;
		st->mem = realloc(st->mem, st->stacksize);
		assert(st->mem != NULL);
	}
	memcpy(st->mem + st->stackpos, data, size);
	st->stackpos = st->stackpos + size;
}

void pop(stack *st, void *data, int size){
	assert(st->stackpos - size >= 0);
	st->stackpos = st->stackpos - size;
	memcpy(data, st->mem + st->stackpos, size);
}

void printRecursive(treeNode *root){
	if(root == NULL) //exit condition
		return;
	printRecursive(root->left);
	printf("%d (%d)\n", root->value, root->count);
	printRecursive(root->right);
}

void print(treeNode *root){
	if(root == NULL) 
		return;
	stack st;
	stackinit(&st);
	int direction;
	treeNode *p = root;
	
	direction = -1;
	do{
		if(direction == -1){
			if(p->left != NULL){
				push(&st, &p, sizeof(p));
				push(&st, &direction, sizeof(direction));
				direction = -1;
				p = p->left;
			}else
				direction = 0;
		}
		if(direction == 0){
			printf("%d (%d)\n", p->value, p->count);
			direction = 1;
		}
		if(direction == 1){
			if(p->right != NULL){
				push(&st, &p, sizeof(p));
				push(&st, &direction, sizeof(direction));
				direction = -1;
				p = p->right;
			}else
				direction = 2;
		}
		if(direction == 2 && st.stackpos > 0){
			pop(&st, &direction, sizeof(direction));
			pop(&st, &p, sizeof(p));
			direction++;
		}
	}while(st.stackpos > 0 || direction < 2);
	free(st.mem);
}

#define MAX 10000

int main(void){
	int i;
	treeNode *root = NULL;
	for(i = 0; i < MAX; i++)
		root = add(rand() % (2 * MAX), root);
	print(root);
	return 0;
}